'use strict';

var jsdom = require('jsdom');
var axios = require('axios');
var puppeteer = require('puppeteer');
var get = require('lodash.get');
var xpath = require('xpath-ts');

exports.EngineType = void 0;
(function (EngineType) {
    EngineType["Html"] = "html";
    EngineType["Spa"] = "spa";
    EngineType["Json"] = "json";
    EngineType["Xml"] = "xml";
})(exports.EngineType || (exports.EngineType = {}));
exports.OperationType = void 0;
(function (OperationType) {
    OperationType["Difference"] = "difference";
    OperationType["Equal"] = "equal";
    OperationType["Contains"] = "contains";
    OperationType["Regex"] = "regex";
})(exports.OperationType || (exports.OperationType = {}));

const DEFAULT_HTTP_REQUESTS_OPTIONS = {
    headers: {
        "Cache-Control": "no-cache",
        "Accept-Encoding": "gzip, deflate",
    },
};

const request$1 = async (url, request = {}) => {
    try {
        const response = await axios.get(url, {
            ...DEFAULT_HTTP_REQUESTS_OPTIONS,
            ...request,
        });
        return response.data;
    }
    catch (error) {
        throw new Error(`Error fetching data from ${url}: ${error.message}`);
    }
};

const operators = {
    difference: (a, b, sensitive) => sensitive ? a !== b : a.toLowerCase() !== b.toLowerCase(),
    equal: (a, b, sensitive) => sensitive ? a === b : a.toLowerCase() === b.toLowerCase(),
    contains: (a, b, sensitive) => sensitive ? a.includes(b) : a.toLowerCase().includes(b.toLowerCase()),
    regex: (a, b, sensitive) => new RegExp(b, sensitive ? "" : "i").test(a),
};
const applyConditions = (value, conditions) => {
    return conditions.some((condition) => {
        const operation = operators[condition.operation];
        return operation
            ? operation(value, condition.value, condition.sensitive || false)
            : false;
    });
};

const getSelectorValue$2 = (element, selector) => {
    const value = selector.attribute
        ? element.getAttribute(selector.attribute) || ""
        : element.textContent?.trim() || "";
    if (selector.conditions && !applyConditions(value, selector.conditions)) {
        return null;
    }
    return value;
};
const extractData$2 = (document, selectors) => {
    const data = {};
    for (const [key, selector] of Object.entries(selectors)) {
        const elements = document.querySelectorAll(selector.selector);
        if (!elements.length) {
            data[key] = "";
        }
        else if (elements.length === 1) {
            data[key] = getSelectorValue$2(elements[0], selector) || "";
        }
        else {
            data[key] = Array.from(elements)
                .map((element) => getSelectorValue$2(element, selector))
                .filter((value) => value !== null);
        }
    }
    return data;
};

const processData$3 = async (url, options) => {
    const { selectors, request } = options;
    const html = await request$1(url, request);
    const dom = new jsdom.JSDOM(html);
    const document = dom.window.document;
    return extractData$2(document, selectors);
};
const scrapeHtml = async (url, { selectors, request }) => {
    const data = await processData$3(url, { selectors, request });
    return data;
};

const autoScroll = async (page) => {
    await page.evaluate(async () => {
        await new Promise((resolve) => {
            let totalHeight = 0;
            let distance = 100;
            let timer = setInterval(() => {
                let scrollHeight = document.body.scrollHeight;
                window.scrollBy(0, distance);
                totalHeight += distance;
                if (totalHeight >= scrollHeight) {
                    clearInterval(timer);
                    resolve(true);
                }
            }, 100);
        });
    });
};
const request = async (url, config = {}) => {
    const browser = await puppeteer.launch({
        executablePath: config?.executablePath
            ? config.executablePath
            : puppeteer.executablePath(),
        ...config,
    });
    const page = await browser.newPage();
    await page.goto(url);
    await autoScroll(page);
    const content = await page.content();
    await browser.close();
    return content;
};

const processData$2 = async (url, options) => {
    const { selectors, request: request$1 } = options;
    const htmlContent = await request(url, request$1);
    const dom = new jsdom.JSDOM(htmlContent);
    const document = dom.window.document;
    return extractData$2(document, selectors);
};
const scrapeSpa = async (url, { selectors, request }) => {
    const data = await processData$2(url, { selectors, request });
    return data;
};

const getSelectorValue$1 = (value, selector) => {
    if (typeof value !== "string") {
        value = JSON.stringify(value);
    }
    if (selector.conditions && !applyConditions(value, selector.conditions)) {
        return null;
    }
    return value;
};
const extractData$1 = (j, selectors) => {
    const data = {};
    if (Array.isArray(j)) {
        return j
            .map((obj) => {
            const data = {};
            for (const [key, selector] of Object.entries(selectors)) {
                const value = get(obj, selector.selector, "");
                data[key] = getSelectorValue$1(value, selector);
            }
            return data;
        })
            .filter((item) => Object.values(item).some((value) => value !== null));
    }
    for (const [key, selector] of Object.entries(selectors)) {
        const value = get(j, selector.selector, "");
        data[key] = getSelectorValue$1(value, selector);
    }
    return data;
};

const processData$1 = (j, { selectors }) => {
    return extractData$1(j, selectors);
};
const scrapeJson = async (url, { selectors, request }) => {
    const j = await request$1(url, request);
    return processData$1(j, { selectors });
};

const getSelectorValue = (values, selector) => {
    if (selector.conditions) {
        values = values.filter((value) => applyConditions(value, selector.conditions));
    }
    return values.length === 1 ? values[0] : values;
};
const extractData = (document, selectors) => {
    const data = {};
    for (const [key, value] of Object.entries(selectors)) {
        const nodes = xpath.evaluate(value.selector, document, null, xpath.XPathResult.ANY_TYPE, null);
        const nodeValues = [];
        let node = nodes.iterateNext();
        while (node) {
            nodeValues.push(node.textContent || "");
            node = nodes.iterateNext();
        }
        data[key] = getSelectorValue(nodeValues, value);
    }
    return data;
};

const processData = async (xml, { selectors }) => {
    const dom = new jsdom.JSDOM(xml, { contentType: "text/xml" });
    const document = dom.window.document;
    return extractData(document, selectors);
};
const scrapeXml = async (url, { selectors, request }) => {
    const xml = await request$1(url, request);
    const data = await processData(xml, { selectors });
    return data;
};

const SCRAPE_TYPES = {
    html: scrapeHtml,
    spa: scrapeSpa,
    json: scrapeJson,
    xml: scrapeXml,
};
const scrape = async ({ url, engine, options, }) => {
    return await SCRAPE_TYPES[engine](url, options);
};

exports.scrape = scrape;
exports.scrapeHtml = scrapeHtml;
exports.scrapeJson = scrapeJson;
exports.scrapeSpa = scrapeSpa;
exports.scrapeXml = scrapeXml;
//# sourceMappingURL=index.js.map
